﻿using System;
using UnityEngine;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.XR;
using MacGruber;
using octopussy;

namespace SPQR
{

  
    public class SPQRHaptics : MVRScript
    {

        //private EventTrigger myEventTrigger;

        protected JSONStorableString myString;
        private int wave = 0;
        private int vibrating = -1;
        int delayVibrate = 5;
        int vibratingLeft = 0;
        int vibratingRight = 0;
        bool stop = false;

        GameObject playerHands;

        JSONStorableFloat collisionThreshold;
        JSONStorableFloat vibrationFactor;
        JSONStorableFloat vibrationDuration;

        public override void Init()
        {
            Utils.OnInitUI(CreateUIElement);


            JSONStorableString logo = new JSONStorableString("logostring", "\n<size=40><b>SPQR Haptics</b></size>");
            UIDynamicTextField logotext = CreateTextField(logo);
            logotext.UItext.supportRichText = true;
            logotext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logotext.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
            logotext.height = 100f;
            logotext.UItext.alignment = TextAnchor.UpperLeft;


            JSONStorableString logor = new JSONStorableString("logospace", "\n<size=30><b>v4</b></size>");
            UIDynamicTextField logortext = CreateTextField(logor, true);
            logortext.UItext.supportRichText = true;
            logortext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logortext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logortext.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
            logortext.height = 100f;
            logortext.UItext.alignment = TextAnchor.UpperRight;


            collisionThreshold = new JSONStorableFloat("Impact Force Required", 0.001f, 0, 3.0f, true);
            collisionThreshold.storeType = JSONStorableParam.StoreType.Full;
            RegisterFloat(collisionThreshold);
            CreateSlider(collisionThreshold, false);

            vibrationFactor = new JSONStorableFloat("Vibration Factor", 0.01f, 0, 0.1f, true);
            vibrationFactor.storeType = JSONStorableParam.StoreType.Full;
            RegisterFloat(vibrationFactor);
            CreateSlider(vibrationFactor, false);

            vibrationDuration = new JSONStorableFloat("Vibration Duration", 0.06f, 0, 0.5f, true);
            vibrationDuration.storeType = JSONStorableParam.StoreType.Full;
            RegisterFloat(vibrationDuration);
            CreateSlider(vibrationDuration, false);






            // myEventTrigger = new EventTrigger(this, "OnTrigger");
            // Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

            // Utils.SetupButton(this, "Test Trigger", TestTrigger, true);
            // Utils.SetupAction(this, "Trigger", Trigger);
            // SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            // SimpleTriggerHandler.LoadAssets();


            Utils.SetupSpacer(this, 50, false);
            Utils.SetupSpacer(this, 50, true);

            // myString = new JSONStorableString("myString", "", UpdateMyString);
            // RegisterString(myString);
            // UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "My String", myString, false);

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            UIUpdate();
        }
        private void OnAtomRename(string oldid, string newid)
        {
            //myEventTrigger.SyncAtomNames();
        }
        private void UIUpdate()
        {
        }


        void Start()
        {
            try
            {
                RefreshColliders();
                stop = false;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Update()
        {
            try
            {
                // myEventTrigger.Update();

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }


        private void Vibrate(bool isLeft, byte bi, float fstrenght)
        {   
            if(isLeft){
                if(vibratingLeft > 0) return;
                vibratingLeft = delayVibrate;
            } else {
                if(vibratingRight > 0) return;
                vibratingRight = delayVibrate;
            }

            byte[] noize = { bi };
            int channel = isLeft ? 1 : 0;

            if (SuperController.singleton.isOVR){
               StartCoroutine(CoroutineVibrate(vibrationDuration.val, isLeft, noize, fstrenght));
            }




        }




       IEnumerator CoroutineVibrate(float totalTime, bool isLeft, byte[] noize , float fstrenght)
        {
            float elapsedTime = 0f;
            while (elapsedTime <= totalTime)
            {
                elapsedTime += Time.deltaTime;

                float ff = fstrenght - (int)UnityEngine.Random.Range(0,5);
                if(ff < 0) ff = 0;

                 byte[] noize2 = {  BitConverter.GetBytes(ff)[0] };
      

                if (isLeft)
                {
                    // SuperController.LogError("LEFT");
                    OVRHaptics.LeftChannel.Preempt(new OVRHapticsClip(noize, 1));
                    
                }
                else
                {
                    OVRHaptics.RightChannel.Preempt(new OVRHapticsClip(noize, 1));
                }
                OVRHaptics.Process();


                yield return null;
            }
            yield break;
    

        }


        private void TestTrigger()
        {
            var jc = this.GetJSON(true, true, false);
        //    SuperController.LogMessage("current : " + jc.ToString().Replace("\"", "\\\""));

        }
        private void Trigger()
        {
            //myEventTrigger.Trigger();
        }

        void FixedUpdate()
        {
            try
            {
                if(vibratingLeft > 0) vibratingLeft--;
                if(vibratingRight > 0) vibratingRight--;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void OnDestroy()
        {   
            stop = true;
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            //myEventTrigger.Remove();
            Utils.OnDestroyUI();

            playerHands = GameObject.Find("Hands");
            if (playerHands)
            {  

                playerHands
                    .GetComponentsInChildren<Rigidbody>()
                    .Where(f => f.name == "R_Finger_Middle_C"
                             || f.name == "L_Finger_Middle_C"
                             || f.name == "R_Finger_Index_C"
                             || f.name == "L_Finger_Index_C"
                             || f.name == "L_Finger_Thumb_C"
                             || f.name == "R_Finger_Thumb_C"
                             || f.name == "L_Finger_Thumb_A"
                             || f.name == "R_Finger_Thumb_A"
                             || f.name == "L_Finger_Ring_C"
                             || f.name == "R_Finger_Pinky_C"
                             || f.name == "L_Finger_Pinky_C"
                             || f.name == "R_Finger_Ring_C"
                             || f.name == "HandPhysical"
                             ).ToList().ForEach(f =>
                             {
                                 f.gameObject.AddComponent<TriggerCollide>().OnCollide -= ObserveSpankTrigger;
                                 f.gameObject.AddComponent<TriggerCollide>().OnCollide -= ObserveSpankTriggerLeft;
                             });
            }
        }



        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                //jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                //myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }



        //////////////////////////////////////////////

        void UpdateMyString(string name)
        {
        }



        public void RefreshColliders()
        {
            playerHands = GameObject.Find("Hands");
            if (playerHands)
            {// only the middle finger is the actual collider

            bool handLeftDone = false;

            playerHands
                    .GetComponentsInChildren<Rigidbody>()
                    .ToList().ForEach(f =>
                             {
                                 if(f.name == "HandPhysical" && !handLeftDone){
                                     handLeftDone = true;
                                 //   f.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveSpankTriggerLeft;
                                 } else {
                                    //f.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveSpankTrigger;
                                 }
                             });


                playerHands
                    .GetComponentsInChildren<Rigidbody>()
                    .Where(f => f.name == "R_Finger_Middle_C"
                             || f.name == "L_Finger_Middle_C"
                             || f.name == "R_Finger_Index_C"
                             || f.name == "L_Finger_Index_C"
                             || f.name == "L_Finger_Thumb_C"
                             || f.name == "R_Finger_Thumb_C"
                             || f.name == "L_Finger_Thumb_A"
                             || f.name == "R_Finger_Thumb_A"
                             || f.name == "L_Finger_Ring_C"
                             || f.name == "R_Finger_Ring_C"
                             || f.name == "L_Finger_Pinky_C"
                             || f.name == "R_Finger_Pinky_C"

                             ).ToList().ForEach(f =>
                             {
                                 f.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveSpankTrigger;
                             });
            }

        }

        void ObserveSpankTriggerLeft(object sender, TriggerEventArgs e){
            // SuperController.LogMessage("ObserveSpankTriggerLeft "+ sender.ToString());
            ObserveSpankTriggerCallback(sender, e, true);
        }
        void ObserveSpankTrigger(object sender, TriggerEventArgs e){
            // SuperController.LogMessage("ObserveSpankTrigger "+ sender.ToString());
            ObserveSpankTriggerCallback(sender, e, false);
        }

        void ObserveSpankTriggerCallback(object sender, TriggerEventArgs e, bool forceLeft)
        {
            if (stop) return;
            bool isLeft = false; 
            if (sender.ToString() == "L_Finger_Middle_C (octopussy.TriggerCollide)")  isLeft = true;
            if (sender.ToString() == "L_Finger_Index_C (octopussy.TriggerCollide)")  isLeft = true;
            if (sender.ToString() == "L_Finger_Thumb_C (octopussy.TriggerCollide)")  isLeft = true;
            if (sender.ToString() == "L_Finger_Thumb_A (octopussy.TriggerCollide)")  isLeft = true;
            if (sender.ToString() == "L_Finger_Ring_C (octopussy.TriggerCollide)")  isLeft = true;
            if (sender.ToString() == "L_Finger_Pinky_C (octopussy.TriggerCollide)")  isLeft = true;
            if(forceLeft) isLeft = true;

            //  SuperController.LogError("!!!!!Receiving event " + e.collider.name);
            if (e.evtType == octopussy.EventType.HIT)
            {
                float level = e.collision.relativeVelocity.magnitude / 3.0f;
                if (level > collisionThreshold.val)
                {
                    if (level > 3) level = 3;

                    //SuperController.LogMessage("collide at high level: " + level + ", threshold: " + collisionThreshold.val);
                    if (e.insideTrigger)
                    {
                        //  SuperController.LogMessage("Collide with trigger");

                    }
                    else
                    {
                        // SuperController.LogError("level: "+level.ToString());

                        float fs = (255 * level / 2) *  vibrationFactor.val;
                        if(fs > 255) fs = 255;
                        byte vibrationLevel = BitConverter.GetBytes(fs)[0] ;
                      //  SuperController.LogMessage("vibrationLevel:" + vibrationLevel.ToString());
                        Vibrate(isLeft, vibrationLevel,fs);

                        // SuperController.LogMessage("Collide with other object");

                    }
                }
                else
                {
                    // SuperController.LogMessage("collide at low level: " + level + ", threshold: " + collisionThreshold.val);
                }
            }
            else
            {

                //  SuperController.LogMessage("Non hit collide");
            }
        }


    }
}
